/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef TAS_REMOTEMATLABPUB_H
#define TAS_REMOTEMATLABPUB_H

#include "controllers/common/utilities/imageviewer/ImageData.h"
#include "controllers/common/acquisition/ImageGeneratable.h"
#include "controllers/common/acquisition/ExperimentData.h"
#include "controllers/common/scanlegacy/GenericScan1D.h"
#include "controllers/common/utilities/CountSpy.h"
#include "NumorMessages.pb.h"
#include <cameo/cameo.h>

namespace tas {

class RemoteMatlabPub : public utilities::ImageData,
						public acquisition::ImageGeneratable,
						public controller::Start {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteMatlabPub(const std::string& name);
	RemoteMatlabPub(const RemoteMatlabPub& controller);
	virtual ~RemoteMatlabPub();

	virtual void postConfiguration();

	void updateNumor();
	void updateCountStatusMessage();

	virtual void start();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<bool> test;
	Property<int32> testNumor;

	ControllerPtr<acquisition::ExperimentData> experimentData;
	ControllerPtr<scan::GenericScan1D> scanController;
	ControllerPtr<utilities::CountSpy> countSpy;

private:
	bool initApplication();
	void sendNumor(int32 numor, bool live);
	void subscriberLoop();
	void sendImageToFluentd(const proto::NumorResponse& response);

	std::string getProposalId();
	std::string getProposalName();
	std::string getInstrumentId();
	std::string getInstrumentName();
	std::string getSampleId();
	std::string getSampleName();
	std::string getCycleId();

	static const std::string MATLAB_APPLICATION;
	static const std::string NUMOR_PUBLISHER;
	static const std::string IMAGE_PUBLISHER;
	static const int32 LOG_TYPE;
	static const int32 LIVE_TYPE;

	std::auto_ptr<cameo::Server> m_server;
	std::auto_ptr<cameo::application::Instance> m_matlabApplication;
	std::auto_ptr<cameo::application::Publisher> m_publisher;
	std::auto_ptr<cameo::application::Subscriber> m_subscriber;
	std::auto_ptr<boost::thread> m_subscriberThread;

	std::string m_nomadSendData;
};

}

#endif
